#pragma once

//#include "../Utils/3DMaths.hpp"   // Using PCH instead


class Camera final {
public:
    Camera();
    ~Camera() {}

    Camera(Camera&&) = default;
    Camera& operator= (Camera&&) = default;
    Camera(Camera const&) = delete;
    Camera& operator= (Camera const&) = delete;

    M3D_VECTOR GetPos() const { return M3D_V4LoadF3(&mPos); }
    M3D_F3 GetPos3f() const { return mPos; }
    M3D_MATRIX GetView() const { return M3D_V4LoadF4x4(&mViewMat); }
    M3D_F4X4 GetView4x4f() const { return mViewMat; }
    M3D_MATRIX GetProj() const { return M3D_V4LoadF4x4(&mProjMat); }
    M3D_F4X4 GetProj4x4f() const { return mProjMat; }

    void SetPosition(const float x, const float y, const float z) { mPos = M3D_F3(x, y, z); }
    void SetPosition(const M3D_F3 v) { mPos = v; }

    void SetFrustrum(float fov, float r, float zn, float zf);
    void UpdateCamView();
    void LookAt(M3D_VECTOR pos, M3D_VECTOR target, M3D_VECTOR worldUp);
    void LookAt(const M3D_F3& pos, const M3D_F3& target, const M3D_F3& up);

private:
    M3D_F4X4 mProjMat = M3D_MIdentity4x4();
    M3D_F4X4 mViewMat = M3D_MIdentity4x4();

    M3D_F3 mPos = {1.5f, 1.5f, 1.5f};
    M3D_F3 mRight = {1.0f, 0.0f, 0.0f};
    M3D_F3 mUp = {0.0f, 1.0f, 0.0f};
    M3D_F3 mLook = {0.0f, 0.0f, 1.0f};

};